module QCD
  use Consts
  implicit none
  
  real(8) :: alphas_mZ, beta(0:3), Lambda_QCD
  integer :: nf = 5
  integer :: alphas_method = 1 ! 1: Alphas_Lambda, 2: Alphas_MSbar, 3: AlphasPDF
  integer :: alphas_order
  real(8) :: mZ = 91.1876d0

  real(8) :: mu_r
  real(8) :: alphas_mur, alphas_muf

contains

  subroutine beta_coeff
    implicit none
    beta(0) = 11d0 - 0.6666667d0 * nf
    beta(1) = 102d0 - 12.66667d0 * nf
    beta(2) = 1428.5d0 - 279.6111d0 * nf + 6.018519d0 * nf**2
    beta(3) = 29242.96d0 - 6946.290d0 * nf + 405.0890d0 * nf**2 + 1.499314d0 * nf**3
  end subroutine beta_coeff

  subroutine InitQCD
    implicit none
    real(8), external :: DZEROX
    call beta_coeff
    Lambda_QCD = DZEROX(0.05d0, 1d0, 1d-8, 10000, Find_Lambda5, 1)
  end subroutine InitQCD

  function Find_Lambda5(Lambda) result (alphas)
    implicit none
    real(8), intent(in) :: Lambda
    real(8) :: alphas
    alphas = Alphas_Lambda_Gen(mZ, Lambda) - alphas_mZ
  end function Find_Lambda5

  function Alphas_Lambda_Gen(scale, Lambda) result (alphas)
    implicit none
    real(8), intent(in) :: scale, Lambda
    real(8) :: alphas
    real(8) :: L, LL
    L = 2d0 * log(scale / Lambda)
    LL = log(L)
    alphas = 1d0
    if (alphas_order > 0) then
       alphas = alphas - beta(1)/beta(0)**2 * LL/L
    endif
    if (alphas_order > 1) then
       alphas = alphas + beta(1)**2/beta(0)**4/L**2 * &
            ( (LL - .5d0)**2 + beta(2)*beta(0)/beta(1)**2  - 5d0/4d0 )
    endif
    if (alphas_order > 2) then
       alphas = alphas + beta(1)**3/beta(0)**6/L**3 * &
            ( -LL**3 + 5d0*LL**2/2d0 + (2d0 - 3d0*beta(0)*beta(2)/beta(1)**2)*LL &
            + beta(0)**2*beta(3)/2d0/beta(1)**3 - .5d0 )
    endif
    alphas = alphas * 4d0*pi/beta(0)/L
  end function Alphas_Lambda_Gen

  function Alphas_Lambda(scale) result (alphas)
    implicit none
    real(8), intent(in) :: scale
    real(8) :: alphas
    alphas = Alphas_Lambda_Gen(scale, Lambda_QCD)
  end function Alphas_Lambda

  function Alphas_MSbar(scale) result (alphas)
    implicit none
    real(8), intent(in) :: scale
    real(8) :: alphas
    real(8) :: alphas0, scale0, a0, v, lv
    alphas0 = alphas_mZ
    scale0 = mZ
    a0 = alphas0/4d0/pi
    v = 1d0 + beta(0) * alphas0/2d0/pi * log(scale/scale0)
    lv = log(v)
    alphas = 1d0
    if (alphas_order > 0) then
       alphas = alphas - a0 * (beta(1)/beta(0)) * lv/v
    endif
    if (alphas_order > 1) then
       alphas = alphas + a0**2 * ( (beta(1)**2/beta(0)**2) * (lv**2 - lv - 1d0 + v) &
            + (beta(2)/beta(0)) * (1d0 - v)) / v**2
    endif
    if (alphas_order > 2) then
       alphas = alphas + a0**3 * ( (beta(1)**3/beta(0)**3) * (2d0*(1d0-v)*lv + 5d0*lv**2/2d0 &
            - lv**3 - .5d0 + v - .5d0*v**2) + (beta(3)/2d0/beta(0)) * (1d0 - v**2) &
            + (beta(1)*beta(2)/beta(0)**2) * (2d0*v*lv - 3d0*lv - v + v**2) )
    endif
    alphas = alphas * alphas0/v
  end function Alphas_MSbar

  function Get_Alphas(scale) result (alphas)
    implicit none 
    real(8), intent(in) :: scale
    real(8) :: alphas
    real(8), external :: AlphasPDF

    select case (alphas_method)
    case (1)
       alphas = Alphas_Lambda(scale)
    case (2)
       alphas = Alphas_MSbar(scale)
    case (3)
       alphas = AlphasPDF(scale)
    end select
  end function Get_Alphas

end module QCD
